#include <Application.h>
#include <Screen.h>
#include <Path.h>
#include "common.h"
#include "app.h"
#include "ctrl.h"
#include "status.h"
#include "pie.h"


// Non-resources :-)
const float kMinWinSize = 275.0;


Win::Win(BRect pieRect):
    BWindow(pieRect, "guido", B_TITLED_WINDOW,
	B_OUTLINE_RESIZE|B_ASYNCHRONOUS_CONTROLS)
{
    BRect r = Bounds();
    r.bottom = r.top + 1.0;
    ctrlView = new CtrlView(r);
    float cvHeight = ctrlView->Bounds().Height();
    AddChild(ctrlView);

    r = Bounds();
    r.top = r.bottom - 1.0;
    statusView = new StatusView(r);
    float svHeight = statusView->Bounds().Height();

    float winHeight = pieRect.Height() + cvHeight + svHeight;
    statusView->MoveTo(0.0, winHeight - svHeight);
    ResizeTo(r.Width(), winHeight);

    AddChild(statusView);

    r = ctrlView->Frame();
    r.top = r.bottom + 1.0;
    r.bottom = r.top + pieRect.Height() - 1.0;
    pieView = new PieView(r);
    AddChild(pieView);

    Show();

    // Note: The following code is semi-broken because BScreen::Frame()
    // returns incorrect dimensions for the G200 in 1152x864 mode.  I reported
    // this bug, and Be said it's not a bug -- the Matrox driver actually uses
    // a resolution of 1152x900 in that mode.  Oh well.
    Lock();
    float extraHeight = ctrlView->Bounds().Height() + statusView->Bounds().Height();
    float minHeight = kMinWinSize + extraHeight;
    float maxHeight = BScreen(this).Frame().Height();
    float maxWidth = maxHeight - extraHeight;
    Unlock();

    SetSizeLimits(kMinWinSize, maxWidth, minHeight, maxHeight);
}


Win::~Win()
{
}


bool
Win::_fixAspectRatio(float *width, float *height)
{
    float ctrlViewHeight = ctrlView->Bounds().Height();
    float statusViewHeight = statusView->Bounds().Height();

    float newPieSize = *height - ctrlViewHeight - statusViewHeight;
    if (*width < newPieSize) {
	newPieSize = *width;
    }
    float newWidth = newPieSize;
    float newHeight = newPieSize + ctrlViewHeight + statusViewHeight;
    if (fabs(*width - newWidth) >= 1.0 || fabs(*height - newHeight) >= 1.0) {
	*width = newWidth;
	*height = newHeight;
	return true;
    }

    return false;
}


void
Win::MessageReceived(BMessage *m)
{
    switch (m->what) {
	case kBtnRescan:
	case kMenuSelectVol:
	    pieView->MessageReceived(m);
	    break;
	case kBtnHelp:
	    be_roster->Launch(&kHelpFileRef);
	    break;
        default:
            BWindow::MessageReceived(m);
            break;
    }
}


void
Win::Zoom(BPoint origin, float width, float height)
{
    if (_fixAspectRatio(&width, &height)) {
	SetZoomLimits(width, height);
    }

    BWindow::Zoom(origin, width, height);
}


void
Win::FrameResized(float width, float height)
{
    if (_fixAspectRatio(&width, &height)) {
	ResizeTo(width, height);
    }
}


bool
Win::QuitRequested()
{
    // Try to save the window parameters to the executable.
    BRect pieRect(Frame());
    pieRect.bottom -= ctrlView->Bounds().Height() + statusView->Bounds().Height();
    BNode appNode(&kAppInfo.ref);
    if (appNode.InitCheck() == B_OK) {
	appNode.WriteAttr(kPieRectAttrName, B_RECT_TYPE, 0, &pieRect, sizeof(BRect));
    }

    be_app->PostMessage(B_QUIT_REQUESTED);
    return true;
}


App::App():
    BApplication(kGuidoSig), _args()
{
    _resources = readResources(kGuidoSig);

    // Try to get the window parameters from the executable.
    BRect pieRect;
    BNode appNode(&kAppInfo.ref);
    if (appNode.InitCheck() == B_OK) {
	if (appNode.ReadAttr(kPieRectAttrName, B_RECT_TYPE, 0, &pieRect,
	    sizeof(BRect)) != sizeof(BRect)) {
	    pieRect.Set(
		50.0, 50.0, 49.0+kDefaultPieSize, 49.0+kDefaultPieSize);
	}
    }

    _win = new Win(pieRect);
}


App::~App()
{
    // Keem up, keem up, everybody everywhere.
    delete _resources;
}


void
App::ArgvReceived(int32 argc, char **argv)
{
    for (int i = 1; i < argc; i++) {
	_args.push_back(argv[i]);
    }
}


void
App::RefsReceived(BMessage *m)
{
    _win->pieView->MessageReceived(m);
}


void
App::ReadyToRun()
{
    if (_args.size() > 0) {
	vector<string>::iterator i = _args.begin();
	while (i != _args.end()) {
	    BEntry entry((*i).c_str(), true);
	    _win->pieView->handleArg(&entry);
	    i++;
	}
    }
}


void
App::MessageReceived(BMessage *m)
{
    switch (m->what) {
	default:
	    BHandler::MessageReceived(m);
	    break;
    }
}


int
main()
{
    new App();
    be_app->Run();
    delete be_app;
    return 0;
}

