#include <StringView.h>
#include <Bitmap.h>
#include <NodeInfo.h>
#include <vector>
#include <pair.h>
#include <stdio.h>
#include <time.h>
#include "info.h"
#include "common.h"


LeftView::LeftView(BRect r, BBitmap *icon):
    BView(r, NULL, B_FOLLOW_NONE, B_WILL_DRAW), _icon(icon)
{
    SetViewColor(kWindowColor);
}


LeftView::~LeftView()
{
    delete _icon;
}


void
LeftView::Draw(BRect r)
{
    float right = Bounds().Width() - kSmallHMargin;
    BRect iconRect(right - 31.0, kSmallVMargin, right, kSmallVMargin + 31.0);
    if (r.Intersects(iconRect)) {
	SetDrawingMode(B_OP_OVER);
	DrawBitmap(_icon, iconRect);
    }
}


InfoWin::InfoWin(BPoint p, FileInfo *f):
    BWindow(BRect(p.x, p.y, p.x, p.y), kEmptyStr, B_FLOATING_WINDOW_LOOK,
	B_NORMAL_WINDOW_FEEL,
	B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE)
{
    typedef pair<string, string> Item;
    typedef vector<Item> InfoList;

    strcpy(strbuf, f->ref.name);
    strcat(strbuf, " info");
    SetTitle(strbuf);

    InfoList info;
    Item item;

    // Size
    scaleByteCount(f->size); // result in global strbuf[]
    if (f->count > 0) {
	// This is a directory.
	char str[64];
	sprintf(str, kInfoInFiles, f->count);
	strcat(strbuf, str);
    }
    info.push_back(Item(kInfoSize, strbuf));

    // Created & modified dates
    BEntry entry(&f->ref);
    time_t t;
    entry.GetCreationTime(&t);
    strftime(strbuf, 64, kInfoTimeFmt, localtime(&t));
    info.push_back(Item(kInfoCreated, strbuf));
    entry.GetModificationTime(&t);
    strftime(strbuf, 64, kInfoTimeFmt, localtime(&t));
    info.push_back(Item(kInfoModified, strbuf));

    // Kind
    BMimeType *type = f->type();
    type->GetShortDescription(strbuf);
    info.push_back(Item(kInfoKind, strbuf));
    delete type;

    // Path
    string path;
    f->getPath(path);
    info.push_back(Item(kInfoPath, path));

    // Icon
    BBitmap *icon = new BBitmap(BRect(0.0, 0.0, 31.0, 31.0), B_CMAP8);
    entry_ref ref;
    entry.GetRef(&ref);
    BNodeInfo::GetTrackerIcon(&ref, icon, B_LARGE_ICON);

    // Compute the window size and add the views.
    BFont smallFont(be_plain_font);
    smallFont.SetSize(10.0);

    struct font_height fh;
    smallFont.GetHeight(&fh);
    float fontHeight = fh.ascent + fh.descent + fh.leading;

    float leftWidth = 32.0;
    float rightWidth = 0.0;
    InfoList::iterator i = info.begin();
    while (i != info.end()) {
	float w = smallFont.StringWidth((*i).first.c_str()) + 2.0 * kSmallHMargin;
	leftWidth = max(leftWidth, w);
	w = smallFont.StringWidth((*i).second.c_str()) + 2.0 * kSmallHMargin;
	rightWidth = max(rightWidth, w);
	i++;
    }

    float winHeight = 32.0 + 4.0 * kSmallVMargin + 5.0 * (fontHeight + kSmallVMargin);
    float winWidth = leftWidth + rightWidth;
    ResizeTo(winWidth, winHeight);

    LeftView *leftView = new LeftView(BRect(0.0, 0.0, leftWidth, winHeight), icon);
    BView *rightView = new BView(
	BRect(leftWidth + 1.0, 0.0, winWidth, winHeight), NULL,
	B_FOLLOW_NONE, B_WILL_DRAW);

    AddChild(leftView);
    AddChild(rightView);

    BStringView *sv = new BStringView(
	BRect(kSmallHMargin, kSmallVMargin, rightView->Bounds().Width(), kSmallVMargin + 30.0),
	NULL, f->ref.name, B_FOLLOW_ALL);
    BFont largeFont(be_plain_font);
    largeFont.SetSize(12.0);
    sv->SetFont(&largeFont);
    rightView->AddChild(sv);

    float y = 32.0 + 4.0 * kSmallVMargin;
    i = info.begin();
    while (i != info.end()) {
	sv = new BStringView(
	    BRect(kSmallHMargin, y, leftView->Bounds().Width(), y + fontHeight),
	    NULL, (*i).first.c_str());
	sv->SetFont(&smallFont);
	sv->SetAlignment(B_ALIGN_RIGHT);
	sv->SetHighColor(kBasePieColor[1]); // arbitrary
	leftView->AddChild(sv);

	sv = new BStringView(
	    BRect(kSmallHMargin, y, rightView->Bounds().Width(), y + fontHeight),
	    NULL, (*i).second.c_str());
	sv->SetFont(&smallFont);
	rightView->AddChild(sv);

	y += fontHeight + kSmallVMargin;
	i++;
    }

    Show();
}


InfoWin::~InfoWin()
{
}


