#ifndef __PIE_H
#define __PIE_H

#include <MenuItem.h>
#include <Bitmap.h>
#include <View.h>
#include <Volume.h>
#include <PopUpMenu.h>
#include <map>
#include <vector>
#include "app.h"
#include "scanner.h"


class AppMenuItem: public BMenuItem
{
    // instance variables
    private:
	int	    _category;
	BBitmap	   *_icon;
	entry_ref   _appRef;
	bool	    _valid;

    // methods
    public:
		AppMenuItem(const char *appSig, int category);
		~AppMenuItem();

	void	GetContentSize(float *width, float *height);
	void	DrawContent();

	int	category() const { return _category; }
	entry_ref *appRef() { return &_appRef; }
	bool	valid() const { return _valid; }
};


class PieView: public BView
{
    // typedefs
    private:
	struct Segment {
	    Segment():
		begin(0.0), end(0.0), info(NULL) { }
	    Segment(float b, float e, FileInfo *i):
		begin(b), end(e), info(i) { }
	    Segment(const Segment &s):
		begin(s.begin), end(s.end), info(s.info) { }
	    ~Segment() { }

	    float begin, end;
	    FileInfo *info;
	};
	typedef vector<Segment> SegmentList;
	typedef map<int, SegmentList> MouseOverInfo;

	typedef map<BVolume*, Scanner*> ScannerMap;

    // instance variables
    private:
	Win	       *_win;
	ScannerMap	_scanners;
	BVolume	       *_currentVolume;
	FileInfo       *_currentDir;
	MouseOverInfo	_mouseOverInfo;
	BPopUpMenu     *_mouseOverMenu;
	BPopUpMenu     *_fileUnavailMenu;
	float		_fontHeight;
	bool		_clicked;
	bool		_dragging;
	BPoint		_whereClicked;
	FileInfo       *_fileClicked;

    // methods
    private:
	void	_showVolume(BVolume *v);
	void	_drawProgressBar(BRect updateRect);
	void	_drawPieChart(BRect updateRect);
	float	_drawDirectory(FileInfo *f, float parentSpan, float beginAngle,
				int colorIdx, int level);
	FileInfo *_fileAt(BPoint p);
	void	_addAppToList(vector<AppMenuItem*> &list, const char *appSig, int category);
	BMenu  *_buildOpenWithMenu(FileInfo *f);
	void	_menu(FileInfo *f, BPoint p);
	void	_launch(FileInfo *f, entry_ref *ref=NULL);

    public:
                PieView(BRect r);
                ~PieView();

	void	handleArg(BEntry *entry);

	void	AttachedToWindow() { _win = (Win *) Window(); }
	void	Draw(BRect updateRect);
        void	MessageReceived(BMessage *m);
	void	MouseDown(BPoint p);
	void	MouseUp(BPoint p);
	void	MouseMoved(BPoint p, uint32 transit, const BMessage *m);
};

#endif // __PIE_H

