#include <NodeInfo.h>
#include <Mime.h>
#include <Path.h>
#include <stdio.h>
#include "snapshot.h"


const char *kPathSepStr = "/";
const char *kFileType = B_FILE_MIME_TYPE;
const char *kDirType = "application/x-vnd.Be-directory";
const char *kVolumeType = "application/x-vnd.Be-volume";


FileInfo::FileInfo():
    pseudo(false), size(0), count(0), parent(NULL), children()
{
}


FileInfo::~FileInfo()
{
    while (children.size() != 0) {
	FileInfo *child = *children.begin();
	if (child != NULL) {
	    delete child;
	}
	children.erase(children.begin());
    }
}


void
FileInfo::getPath(string &path)
{
    if (pseudo) {
	path.assign(ref.name);
    }
    else {
	BEntry entry(&ref, true);
	BPath pathObj(&entry);
	path.assign(pathObj.Path());
    }
}


FileInfo *
FileInfo::findChild(const char *name)
{
    vector<FileInfo *>::iterator i = children.begin();
    while (i != children.end()) {
	if (strcmp((*i)->ref.name, name) == 0) {
	    return *i;
	}
	i++;
    }

    return NULL;
}


BMimeType *
FileInfo::type()
{
    char mimeStr[B_MIME_TYPE_LENGTH] = { '\0' };
    if (parent == NULL) {
	// This is the volume's root directory; treat it as a volume type.
	strcpy(mimeStr, kVolumeType);
    }
    else {
	// Get the MIME type from the registrar.
	BNode node(&ref);
	if (node.InitCheck() == B_OK) {
	    BNodeInfo nodeInfo(&node);
	    if (nodeInfo.InitCheck() == B_OK) {
		status_t s = nodeInfo.GetType(mimeStr);
		if (s != B_OK && children.size() > 0) {
		    if (s == B_ENTRY_NOT_FOUND) {
			// This status appears to be returned only for files on
			// BFS volumes (e.g., CDFS volumes return B_BAD_VALUE).
			//nodeInfo.SetType(kDirType);
		    }
		    strcpy(mimeStr, kDirType);
		}
	    }
	}
    }

    if (strlen(mimeStr) == 0) {
	strcpy(mimeStr, kFileType);
    }

    return new BMimeType(mimeStr);
}


VolSnapshot::VolSnapshot(BVolume *v)
{
    char nameBuffer[NAME_MAX];
    v->GetName(nameBuffer);
    name = nameBuffer;

    capacity = v->Capacity();
    freeBytes = v->FreeBytes();
    rootDir = NULL;
    freeSpace = NULL;
}


VolSnapshot::~VolSnapshot()
{
    if (rootDir != NULL) {
	delete rootDir;
    }
    if (freeSpace != NULL) {
	delete freeSpace;
    }
}

