#include <NodeMonitor.h>
#include <StringView.h>
#include <Box.h>
#include <math.h>
#include <stdio.h>
#include "common.h"
#include "status.h"


StatusView::StatusView(BRect r):
    BView(r, NULL, B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM, B_WILL_DRAW),
    _currentFileInfo(NULL)
{
    SetViewColor(kWindowColor);

    struct font_height fh;
    be_plain_font->GetHeight(&fh);
    float plainHeight = fh.ascent + fh.descent + fh.leading;
    float fixedHeight = fh.ascent + fh.descent + fh.leading;
    ResizeTo(Bounds().Width(), 2.0 * kSmallVMargin + max(plainHeight, fixedHeight));

    BRect r = Bounds();

    r.top += kSmallVMargin;

    // Size display
    r.right -= kSmallHMargin;
    r.left = r.right - StringWidth("9999.99 GB");
    _size = new BStringView(r, NULL, kEmptyStr,
	B_FOLLOW_RIGHT|B_FOLLOW_TOP_BOTTOM);
    AddChild(_size);

    // Vertical divider
    r.right = r.left - kSmallHMargin;
    r.left = r.right - 1.0;
    r.top -= 5.0; r.bottom += 5.0;
    AddChild(new BBox(r, NULL, B_FOLLOW_RIGHT|B_FOLLOW_TOP_BOTTOM));
    r.top += 5.0; r.bottom -= 5.0;

    // Count display
    sprintf(strbuf, kManyFiles, 999999);
    r.right = r.left - kSmallHMargin;
    r.left = r.right - (StringWidth(strbuf) + kSmallHMargin);
    _count = new BStringView(r, NULL, kEmptyStr,
	B_FOLLOW_RIGHT|B_FOLLOW_TOP_BOTTOM);
    AddChild(_count);

    // Vertical divider
    r.right = r.left - kSmallHMargin;
    r.left = r.right - 1.0;
    r.top -= 5.0; r.bottom += 5.0;
    AddChild(new BBox(r, NULL, B_FOLLOW_RIGHT|B_FOLLOW_TOP_BOTTOM));
    r.top += 5.0; r.bottom -= 5.0;

    // Path display
    r.right = r.left - kSmallHMargin;
    r.left = kSmallHMargin;
    _path = new BStringView(r, NULL, kEmptyStr,
	B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP_BOTTOM);
    AddChild(_path);

    // Horizontal divider
    r = Bounds();
    r.bottom = r.top + 1.0;
    r.left -= 5.0; r.right += 5.0;
    AddChild(new BBox(r, NULL, B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM, B_WILL_DRAW));
}


StatusView::~StatusView()
{
}


void
StatusView::show(FileInfo *f)
{
    if (f == _currentFileInfo) {
	return;
    }

    _currentFileInfo = f;

    if (f == NULL) {
	_path->SetText(kEmptyStr);
	_size->SetText(kEmptyStr);
	_count->SetText(kEmptyStr);
	return;
    }

    if (!f->pseudo) {
	BNode node(&f->ref);
	if (node.InitCheck() != B_OK) {
	    _path->SetText(kStrUnavail);
	    _size->SetText(kEmptyStr);
	    _count->SetText(kEmptyStr);
	    return;
	}
    }

    float viewWidth = _path->Bounds().Width();
    string path;
    f->getPath(path);
    const char *c = path.c_str();
    if (StringWidth(c) <= viewWidth) {
	_path->SetText(c);
    }
    else {
	while (StringWidth(c) + StringWidth(B_UTF8_ELLIPSIS) > viewWidth) {
	    c++;
	}
	string text(B_UTF8_ELLIPSIS);
	text.append(c);
	_path->SetText(text.c_str());
    }

    _size->SetText(scaleByteCount(f->size));

    if (f->count > 0) {
	sprintf(strbuf, (f->count == 1) ? kOneFile : kManyFiles, f->count);
	_count->SetText(strbuf);
    }
    else {
	_count->SetText(kEmptyStr);
    }
}


